/*  Programa ESTAT.C
*
*  Este programa demonstra os processos de escrita e leitura em arquivos txt, usando vrias funes de manipulao de arquivos.
*  Tambm demonstra utilizao de macros e ponteiros para char (passagem de strings para funes).
*
*  Setembro 2006 Marcos Portnoi
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

#define TAM 1000 //tamanho do conjunto de dados
#define IN_FILE "dados.txt" //arquivo de entrada de dados, que ser gerado aleatoriamente pelo programa
#define OUT_FILE "estatistica.txt" //arquivo onde as estatsticas sero gravadas

void geraDados(char *inFile);
void calcEstat(char *inFile, char *outFile);

int main(int argc, char *argv[]) {
	char choice;
	
	do {
	   	printf("\n\nDemonstrativo de Acesso a Arquivo\n\n");
	   	printf("(1)  Gerar arquivo \"%s\" com numeros aleatorios\n", IN_FILE);
	   	printf("(2)  Ler arquivo \"%s\" e calcular media e desvio padrao\n", IN_FILE);
	   	printf("(0)  Sair do programa\n");
 		printf("\n  Escolha:  ");
	   	flushall(); //limpa o buffer de teclado antes da leitura; experimentar comentar esta linha e ver o que acontece
		scanf("%c", &choice);

		switch (choice) {
			case '1':
				geraDados(IN_FILE, TAM);
				break;

			case '2':
				calcEstat(IN_FILE, OUT_FILE);
				break;

			case '0':
				return 0;
				break;

			default:
				printf("\n\nOpcao invalida!\n\n");
				break;
		}
	} while (1);
}

/* FUNO DE GERAO DE ARQUIVO COM DADOS NUMRICOS
*
*  Esta funo gera um arquivo .txt contendo nmeros aleatrios, um em cada linha.
*  A quantidade de nmeros gerados  recebida tambm como parmetro.
*/
void geraDados(char *inFile, int n) {
	double num;
	int i;
	FILE *fp;

	fp=fopen(inFile, "w"); //se arquivo j existir, ele ser apagado
	srand((unsigned int)time(NULL)); //alimenta gerador de nmeros aleatrios com semente vinda do relgio interno do sistema; este valor deve ser do tipo unsigned int, razo do cast

	for (i=0; i<n; i++) {
		num = (double)(rand()%10000)/100;
		fprintf(fp, "%f\n", num); //gera nmero aleatrio entre 0 e 100 com duas casas decimais
	}
	fclose(fp); //fecha arquivo
}

/* FUNO DE LEITURA DE ARQUIVO DE DADOS E GERAO DE ESTATSTICAS
*
*  Esta funo l o arquivo indicado por inFile, que deve estar no diretrio do executvel e conter um nmero do tipo float por linha.  O arquivo pode conter
*  qualquer quantidade de nmeros.  A funo calcular a mdia e aps far nova leitura do arquivo para calcular o desvio padro amostral.
*  As medidas estatsticas sero gravadas no arquivo indicado por outFile.
*/
void calcEstat(char *inFile, char *outFile) {
	int i=0;
	double num, media=0, s=0;
	FILE *ent, *sai;

	ent = fopen(inFile, "r");
	sai = fopen(outFile, "w");

	if (ent==NULL) {
		printf("Erro:  arquivo dados.txt nao existe.  Crie-o primeiro.");
		return;
	}

	while (fscanf(ent, "%lf", &num)!= EOF) { //leia at o final do arquivo; se no houver mais dados, fscanf retornar EOF
		i++; //incrementa contador de elementos do arquivo inFile; isto ser usado na gerao de estatsticas
		media=media+num; //acumula somatrio de cada nmero
	}
	fprintf(sai, "Estatisticas do Conteudo do Arquivo %s\n\n", inFile);
	fprintf(sai, "Quantidade de dados:  %d\n\n", i);
	fprintf(sai, "Somatorio:  %f\n", media);
	media=media/i; //finaliza clculo da mdia
	fprintf(sai, "Media:  %f\n", media);
	rewind(ent); //retorna o ponteiro de leitura do arquivo inFile para o incio, para clculo agora do desvio padro amostral
	while (fscanf(ent, "%lf", &num) != EOF) {
		s=s+((num-media)*(num-media)); //acumula somatrio de (num-media)^2
	}
	fprintf(sai, "Somatorio quadrado dos desvios:  %f\n", s);
	s=sqrt(s/(i-1)); //finaliza clculo do desvio padro amostral
	fprintf(sai, "Desvio Padrao:  %f\n", s);
	printf("\nArquivo %s gerado.\n", outFile);
	fclose(ent);
	fclose(sai);
}